"""\
This program creates an egg package (called update_NAME-1.0.egg),
for uptaing the package NAME.
"""
import os
import zipfile
from compiler import compileFile
from os.path import join
from optparse import OptionParser



def build_egg(name):
    py_module = 'update_%s.py' % name

    code = open('update_NAME.py').read()
    code = code.replace('@NAME@', name)
    open(py_module, 'w').write(code)
    compileFile(py_module)

    z = zipfile.ZipFile(join('update_%s-1.0.egg' % name),
                        'w', zipfile.ZIP_DEFLATED)
    for ext in ('', 'c'):
        z.write(py_module + ext)

    data = open('appinst.dat').read()
    data = data.replace('@NAME@', name)
    z.writestr('EGG-INFO/inst/appinst.dat', data)

    z.writestr('EGG-INFO/entry_points.txt',
               '[console_scripts]\n'
               'update-%s = update_%s:main\n' % (name, name))
    z.write('update.ico', 'EGG-INFO/inst/update.ico')
    z.write('update.icns', 'EGG-INFO/inst/update.icns')
    z.close()

    for ext in ('', 'c'):
        os.unlink(py_module + ext)


def main():
    p = OptionParser(usage="usage: %prog [options] NAME",
                     description=__doc__)

    opts, args = p.parse_args()

    if len(args) != 1:
        p.error("exactly one argument expected, try -h")

    build_egg(args[0])


if __name__ == '__main__':
    main()



import bz2
import re
import sys
import time
import hashlib
import urlparse
import urllib2
from cStringIO import StringIO
from os.path import abspath, expanduser, getmtime, isfile, join

from egginst import name_version_fn
from egginst.utils import human_bytes
from enstaller import __version__
from enstaller.verlib import NormalizedVersion, IrrationalVersionError


PY_VER = '%i.%i' % sys.version_info[:2]


def abs_expanduser(path):
    return abspath(expanduser(path))


def canonical(s):
    """
    return the canonical representations of a project name
    """
    # eventually (once Python 2.6 repo eggs are no longer supported), this
    # function should only return s.lower()
    s = s.lower()
    s = s.replace('-', '_')
    if s == 'tables':
        s = 'pytables'
    return s


def cname_fn(fn):
    return canonical(fn.split('-')[0])


def comparable_version(version):
    """
    Given a version string (e.g. '1.3.0.dev234'), return an object which
    allows correct comparison. For example:
        comparable_version('1.3.10') > comparable_version('1.3.8')  # True
    whereas:
        '1.3.10' > '1.3.8'  # False
    """
    try:
        # This hack makes it possible to use 'rc' in the version, where
        # 'rc' must be followed by a single digit.
        ver = version.replace('rc', '.dev99999')
        return NormalizedVersion(ver)
    except IrrationalVersionError:
        # If obtaining the RationalVersion object fails (for example for
        # the version '2009j'), simply return the string, such that
        # a string comparison can be made.
        return version


def md5_file(path):
    """
    Returns the md5sum of the file (located at `path`) as a hexadecimal
    string of length 32.
    """
    fi = open(path, 'rb')
    h = hashlib.new('md5')
    while True:
        chunk = fi.read(65536)
        if not chunk:
            break
        h.update(chunk)
    fi.close()
    return h.hexdigest()


def open_with_auth(url):
    """
    Open a urllib2 request, handling HTTP authentication
    """
    import config

    scheme, netloc, path, params, query, frag = urlparse.urlparse(url)
    assert not query
    auth, host = urllib2.splituser(netloc)
    if auth:
        auth = urllib2.unquote(auth).encode('base64').strip()
    elif 'enthought.com/repo/' in url and 'repo/pypi/eggs/' not in url:
        auth = config.get('EPD_auth')
        if auth is None:
            userpass = config.get('EPD_userpass')
            if userpass:
                auth = userpass.encode('base64').strip()

    if auth:
        new_url = urlparse.urlunparse((scheme, host, path,
                                       params, query, frag))
        request = urllib2.Request(new_url)
        request.add_header("Authorization", "Basic " + auth)
    else:
        request = urllib2.Request(url)
    request.add_header('User-Agent', 'enstaller/%s' % __version__)
    return urllib2.urlopen(request)


def write_data_from_url(fo, url, md5=None, size=None):
    """
    Read data from the url and write to the file handle fo, which must be
    open for writing.  Optionally check the MD5.  When the size in bytes
    is provided, a progress bar is displayed using the download/copy.
    """
    if size:
        sys.stdout.write('%9s [' % human_bytes(size))
        sys.stdout.flush()
        n = cur = 0

    if url.startswith('file://'):
        path = url[7:]
        fi = open(path, 'rb')
    elif url.startswith(('http://', 'https://')):
        try:
            fi = open_with_auth(url)
        except urllib2.HTTPError as e:
            sys.stderr.write(str(e) + '\n')
            if '401' in str(e):
                sys.stderr.write("""\
Please make sure you are using the correct authentication.
Use "enpkg --userpass" to update authentication in configuration file.
""")
                sys.exit(1)
    else:
        sys.exit("Error: invalid url: %r" % url)

    h = hashlib.new('md5')

    if size and size < 16384:
        buffsize = 1
    else:
        buffsize = 256

    while True:
        chunk = fi.read(buffsize)
        if not chunk:
            break
        fo.write(chunk)
        if md5:
            h.update(chunk)
        if not size:
            continue
        n += len(chunk)
        if float(n) / size * 64 >= cur:
            sys.stdout.write('.')
            sys.stdout.flush()
            cur += 1

    if size:
        sys.stdout.write(']\n')
        sys.stdout.flush()

    fi.close()

    if md5 and h.hexdigest() != md5:
        sys.stderr.write("FATAL ERROR: Data received from\n\n"
                         "    %s\n\n"
                         "is corrupted.  MD5 sums mismatch.\n" % url)
        fo.close()
        sys.exit(1)

# -----------------------------------------------------------------

repo_pat = re.compile(r'/repo/([^\s/]+/[^\s/]+)/')
def shorten_repo(repo):
    m = repo_pat.search(repo)
    if m:
        return m.group(1)
    else:
        res = repo.replace('http://', '').replace('https://', '')
        return res.replace('.enthought.com', '')


def get_installed_info(prefix, cname):
    """
    return a dictionary with information about the package specified by the
    canonical name found in prefix, or None if the package is not found
    """
    meta_dir = join(prefix, 'EGG-INFO', cname)
    meta_txt = join(meta_dir, '__egginst__.txt')
    if not isfile(meta_txt):
        return None

    d = {}
    execfile(meta_txt, d)
    res = {}
    res['egg_name'] = d['egg_name']
    res['name'], res['version'] = name_version_fn(d['egg_name'])
    res['mtime'] = time.ctime(getmtime(meta_txt))
    res['meta_dir'] = meta_dir

    meta2_txt = join(meta_dir, '__enpkg__.txt')
    if isfile(meta2_txt):
        d = {}
        execfile(meta2_txt, d)
        res['repo'] = shorten_repo(d['repo'])
    return res


def get_info():
    """
    returns a dict mapping canonical project names to spec structures
    containing additional meta-data of the project which is not contained
    in the index-depend data
    """
    from indexed_repo.metadata import parse_index
    import config

    url = config.get('info_url')
    faux = StringIO()
    write_data_from_url(faux, url)
    index_data = faux.getvalue()
    faux.close()

    if url.endswith('.bz2'):
        index_data = bz2.decompress(index_data)

    res = {}
    for name, data in parse_index(index_data).iteritems():
        d = {}
        exec data.replace('\r', '') in d
        cname = canonical(name)
        res[cname] = {}
        for var_name in ('name', 'homepage', 'doclink', 'license',
                         'summary', 'description'):
            res[cname][var_name] = d[var_name]
    return res


def get_available():
    """
    return a dict mapping canonical project names to versions which
    are available in the subscriber repositories
    """
    import plat
    import config

    url = '%savailable/%s.txt' % (config.pypi_url, plat.custom_plat)
    faux = StringIO()
    write_data_from_url(faux, url)
    data = faux.getvalue()
    faux.close()

    res = {}
    for line in data.splitlines():
        line = line.strip()
        if line:
            parts = line.split()
            cname = parts[0]
            versions = parts[1:]
            res[cname] = versions
    return res



import sys
from collections import defaultdict

import egginst

from enstaller.indexed_repo import Chain, Req, dist_naming
from enstaller.utils import get_installed_info, comparable_version
import enstaller.config as config


def get_status():
    # the result is a dict mapping cname to ...
    res = {}
    for cname in egginst.get_installed_cnames(sys.prefix):
        d = defaultdict(str)
        info = get_installed_info(sys.prefix, cname)
        if info is None:
            continue
        d.update(info)
        res[cname] = d

    c = Chain(config.get('IndexedRepos'))

    for cname in c.groups.iterkeys():
        dist = c.get_dist(Req(cname))
        if dist is None:
            continue
        repo, fn = dist_naming.split_dist(dist)
        n, v, b = dist_naming.split_eggname(fn)
        if cname not in res:
            d = defaultdict(str)
            d['name'] = n
            res[cname] = d
        res[cname]['a-egg'] = fn
        res[cname]['a-ver'] = '%s-%d' % (v, b)

    def vb_egg(fn):
        try:
            n, v, b = dist_naming.split_eggname(fn)
            return comparable_version(v), b
        except:
            return None

    for d in res.itervalues():
        if d['egg_name']:                    # installed
            if d['a-egg']:
                if vb_egg(d['egg_name']) >= vb_egg(d['a-egg']):
                    d['status'] = 'up-to-date'
                else:
                    d['status'] = 'updateable'
            else:
                d['status'] = 'installed'
        else:                                # not installed
            if d['a-egg']:
                d['status'] = 'installable'
    return res


if __name__ == '__main__':
    for v in get_status().itervalues():
        print '%(name)-20s %(version)16s %(a-ver)16s %(status)12s' % v



import os
import sys
from collections import defaultdict
from os.path import basename, join, isdir, isfile, normpath



MODULE_EXTENSIONS = ('.pyd', '.so', '.py', '.pyw', '.pyc', 'pyo')
MODULE_EXTENSIONS_SET = set(MODULE_EXTENSIONS)


def stripped_lines(path):
    for line in open(path):
        line = line.strip()
        if not line or line.startswith('#'):
            continue
        yield line


def is_namespace(dir_path):
    modules = set()
    for fn in os.listdir(dir_path):
        path = join(dir_path, fn)
        if isfile(path):
            name, ext = os.path.splitext(basename(path))
            if ext in MODULE_EXTENSIONS_SET:
                modules.add(name)

    return modules == set(['__init__'])


def create_hooks_dir(dir_path, namespace=''):
    reg = {}
    modules = defaultdict(set)
    pth = []
    for fn in os.listdir(dir_path):
        if '-' in fn:
            continue

        path = join(dir_path, fn)
        if isdir(path):
            reg[namespace + fn] = path
            if is_namespace(path):
                add_reg, dummy = create_hooks_dir(path, namespace + fn + '.')
                reg.update(add_reg)

        elif isfile(path):
            name, ext = os.path.splitext(basename(path))
            if ext in MODULE_EXTENSIONS_SET:
                modules[name].add(ext)

            if ext == '.pth':
                for line in stripped_lines(path):
                    pth.append(normpath(join(dir_path, line)))

    for name, exts in modules.iteritems():
        if name == '__init__':
            continue
        for mext in MODULE_EXTENSIONS:
            if mext in exts:
                reg[namespace + name] = join(dir_path, name + mext)
                break

    return reg, pth


def create_file(egg):
    reg, pth = create_hooks_dir(egg.pkg_dir)

    fo = open(egg.registry_txt, 'w')
    fo.write('# pkg: %s\n' % basename(egg.pkg_dir))
    for kv in reg.iteritems():
        fo.write('%s  %s\n' % kv)
    for p in pth:
        fo.write('-pth-  %s\n' % p)
    fo.close()


def collect(packages, path):
    """
    collects the EGG-INFO/registry.txt files for `packages` and writes them
    to a single registry file at `path`
    """
    fo = open(path, 'w')
    for pkg in packages:
        path = join(sys.prefix, 'pkgs', pkg, 'EGG-INFO', 'registry.txt')
        fo.write(open(path).read())
    fo.close()



import random
import unittest

from egginst.main import name_version_fn
from enstaller.utils import canonical, cname_fn, comparable_version


class TestUtils(unittest.TestCase):

    def test_canonical(self):
        for name, cname in [
            ('NumPy', 'numpy'),
            ('MySql-python', 'mysql_python'),
            ('Python-dateutil', 'python_dateutil'),
            ]:
            self.assertEqual(canonical(name), cname)

    def test_cname_fn(self):
        self.assertEqual(cname_fn('VTK-5.4.2-1.egg'), 'vtk')

    def test_naming(self):
        for fn, name, ver, cname in [
            ('NumPy-1.5-py2.6-win32.egg', 'NumPy', '1.5-py2.6-win32', 'numpy'),
            ('NumPy-1.5-2.egg', 'NumPy', '1.5-2', 'numpy'),
            ('NumPy-1.5.egg', 'NumPy', '1.5', 'numpy'),
            ]:
            self.assertEqual(name_version_fn(fn), (name, ver))
            self.assertEqual(cname_fn(fn), cname)
            self.assertEqual(canonical(name), cname)

    def test_comparable_version(self):
        for versions in (
            ['1.0.4', '1.2.1', '1.3.0b1', '1.3.0', '1.3.10',
             '1.3.11.dev7', '1.3.11.dev12', '1.3.11.dev111',
             '1.3.11', '1.3.143',
             '1.4.0.dev7749', '1.4.0rc1', '1.4.0rc2', '1.4.0'],
            ['2008j', '2008k', '2009b', '2009h', '2010b'],
            ['0.99', '1.0a2', '1.0b1', '1.0rc1', '1.0', '1.0.1'],
            ['2.0.8', '2.0.10', '2.0.10.1', '2.0.11'],
            ['0.10.1', '0.10.2', '0.11.dev1324', '0.11'],
            ):
            org = list(versions)
            random.shuffle(versions)
            versions.sort(key=comparable_version)
            self.assertEqual(versions, org)


if __name__ == '__main__':
    unittest.main()



import sys
from os.path import join
from subprocess import check_call

from egginst.utils import bin_dir_name
from enstaller.indexed_repo import Chain, dist_naming
from enstaller.main import get_installed_info
from enstaller import config


config.get_path = lambda: config.SYSTEM_CONFIG_PATH


name = '@NAME@'
enpkg_bin = join(sys.prefix, bin_dir_name, 'enpkg')


def welcome():
    print """\
************************************************************************
*              Welcome to the %s update program. %s    *
************************************************************************
""" % (name, ' ' * (20-len(name)))

def quit():
    raw_input("Hit enter to quit> ")
    sys.exit()

def get_info():
    res = get_installed_info(sys.prefix, name)
    if res is None:
        print "Error: %r does not appear to be installed" % name
        quit()
    return res

def main():
    welcome()
    info = get_info()
    curr_ver = dist_naming.split_eggname(info['egg_name'])[1]
    print "Currently installed version:  %s" % curr_ver
    print "               installed on:  %s" % info['mtime']
    print

    conf = config.read()
    c = Chain(conf['IndexedRepos'])
    versions = c.list_versions(name)
    if len(versions) == 0:
        print "Error: no versions of %r available" % name
        quit()

    while True:
        print "Available versions:"
        print ', '.join(versions)
        print """
You have the following options:
  - press return to update to the latest available version %s
  - enter the version to update (or downgrade) to, e.g. %r
  - enter 'q' to quit (without changing anything)
""" % (versions[-1], versions[0])
        inp = raw_input('> ').strip()
        if inp.lower() in ('q', 'quit'):
            sys.exit()

        update_to = inp or versions[-1]
        if update_to in versions:
            print "Updating to: %s" % update_to
            check_call([enpkg_bin, '--sys-config', name, update_to])
            quit()
        print "You have entered %r, which is not an available version" % inp


if __name__ == '__main__':
    main()



# Author: Ilan Schnell <ischnell@enthought.com>
"""\
enstaller is a managing tool for egginst-based installs, and the CLI is
called enpkg which calls out to egginst to do the actual install.
enpkg can access distributions from local and HTTP repositories.
"""
import os
import re
import sys
import string
import subprocess
import textwrap
from os.path import basename, isdir, isfile, join
from optparse import OptionParser

import egginst
from egginst.utils import bin_dir_name, rel_site_packages, pprint_fn_action

import config
import history
from proxy.api import setup_proxy
from utils import (canonical, cname_fn, get_info, comparable_version,
                   shorten_repo, get_installed_info, get_available)
from indexed_repo import (Chain, Req, add_Reqs_to_spec, filename_as_req,
                          spec_as_req, parse_data, dist_naming)


# global options variables
prefix = None
dry_run = None
verbose = None

c = None
def set_chain():
    global c
    c = Chain(config.get('IndexedRepos'), verbose)


def print_path():
    prefixes = [sys.prefix]
    if prefix != sys.prefix:
        prefixes.insert(0, prefix)
    print "Prefixes:"
    for p in prefixes:
        print '    %s%s' % (p, ['', ' (sys)'][p == sys.prefix])
    print

    cmd = ('export', 'set')[sys.platform == 'win32']
    print "%s PATH=%s" % (cmd, os.pathsep.join(
                                 join(p, bin_dir_name) for p in prefixes))
    if prefix != sys.prefix:
        print "%s PYTHONPATH=%s" % (cmd, join(prefix, rel_site_packages))

    if sys.platform != 'win32':
        if sys.platform == 'darwin':
            name = 'DYLD_LIBRARY_PATH'
        else:
            name = 'LD_LIBRARY_PATH'
        print "%s %s=%s" % (cmd, name, os.pathsep.join(
                                 join(p, 'lib') for p in prefixes))


def egginst_subprocess(pkg_path, remove):
    # only used on Windows
    path = join(sys.prefix, bin_dir_name, 'egginst-script.py')
    args = [sys.executable, path, '--prefix', prefix]
    if dry_run:
        args.append('--dry-run')
    if remove:
        args.append('--remove')
    if config.get('noapp'):
        args.append('--noapp')
    args.append(pkg_path)
    if verbose:
        print 'CALL: %r' % args
    subprocess.call(args)


def egginst_remove(pkg):
    fn = basename(pkg)
    if (sys.platform == 'win32'  and
            fn.lower().startswith(('appinst-', 'pywin32-'))):
        if verbose:
            print "Starting subprocess:"
        egginst_subprocess(pkg, remove=True)
        return
    pprint_fn_action(fn, 'removing')
    if dry_run:
        return
    ei = egginst.EggInst(pkg, prefix, noapp=config.get('noapp'))
    ei.remove()


def egginst_install(dist):
    repo, fn = dist_naming.split_dist(dist)
    pkg_path = join(config.get('local'), fn)
    if (sys.platform == 'win32'  and
            fn.lower().startswith(('appinst-', 'pywin32-'))):
        if verbose:
            print "Starting subprocess:"
        egginst_subprocess(pkg_path, remove=False)
        return
    pprint_fn_action(fn, 'installing')
    if dry_run:
        return
    ei = egginst.EggInst(pkg_path, prefix, noapp=config.get('noapp'))
    ei.install()
    info = get_installed_info(prefix, cname_fn(fn))
    path = join(info['meta_dir'], '__enpkg__.txt')
    fo = open(path, 'w')
    fo.write("repo = %r\n" % repo)
    fo.close()


def check_write():
    if not isdir(prefix):
        os.makedirs(prefix)
    path = join(prefix, 'hello.txt')
    try:
        open(path, 'w').write('Hello World!\n')
    except:
        print "ERROR: Could not write simple file into:", prefix
        sys.exit(1)
    finally:
        if isfile(path):
            os.unlink(path)


def print_installed_info(cname):
    info = get_installed_info(prefix, cname)
    if info is None:
        print "%s is not installed" % cname
    else:
        print "%(egg_name)s was installed on: %(mtime)s" % info

    if prefix == sys.prefix:
        return

    info = get_installed_info(sys.prefix, cname)
    if info is None:
        print "%s is not installed in sys.prefix" % cname
    else:
        print "%(egg_name)s was installed in sys.prefix on: %(mtime)s" % info


def info_option(cname):
    info = get_info()
    if info and cname in info:
        spec = info[cname]
        print "Name    :", spec['name']
        print "License :", spec['license']
        print "Summary :", spec['summary']
        print
        for line in textwrap.wrap(' '.join(spec['description'].split()), 77):
            print line
    print
    print "In repositories:"
    displayed = set()
    for dist in c.iter_dists(Req(cname)):
        repo = dist_naming.repo_dist(dist)
        if repo not in displayed:
            print '    %s' % repo
            displayed.add(repo)
    print

    dist = c.get_dist(Req(cname))
    if dist:
        reqs = set(r.name for r in c.reqs_dist(dist))
        print "Requirements: %s" % ', '.join(sorted(reqs))

    print "Available versions: %s" % ', '.join(c.list_versions(cname))
    print_installed_info(cname)


def print_installed(prefix, pat=None):
    fmt = '%-20s %-20s %s'
    print fmt % ('Project name', 'Version', 'Repository')
    print 60 * '='
    for cname in egginst.get_installed_cnames(prefix):
        if pat and not pat.search(cname):
            continue
        info = get_installed_info(prefix, cname)
        if info is None:
            continue
        print fmt % (info['name'], info['version'], info.get('repo', '-'))


def list_option(pat):
    print "sys.prefix:", sys.prefix
    print_installed(sys.prefix, pat)
    if prefix == sys.prefix:
        return
    print
    print "prefix:", prefix
    print_installed(prefix, pat)


def whats_new():
    fmt = '%-25s %-15s %s'
    print fmt % ('Name', 'installed', 'available')
    print 60* "="

    inst = set(egginst.get_installed(sys.prefix))
    if prefix != sys.prefix:
        inst |= set(egginst.get_installed(prefix))

    something_new = False
    for egg_name in inst:
        if not dist_naming.is_valid_eggname(egg_name):
            continue
        in_n, in_v, in_b = dist_naming.split_eggname(egg_name)
        dist = c.get_dist(Req(in_n))
        if dist is None:
            continue
        av_v = c.index[dist]['version']
        if (av_v != in_v and
                    comparable_version(av_v) > comparable_version(in_v)):
            print fmt % (in_n, in_v, av_v)
            something_new = True

    if not something_new:
        print "no new version of any installed package is available"


def search(pat=None):
    """
    Print the distributions available in a repo, i.e. a "virtual" repo made
    of a chain of (indexed) repos.
    """
    fmt = "%-25s %-15s %s"
    print fmt % ('Project name', 'Versions', 'Repository')
    print 55 * '-'

    for name in sorted(c.groups.keys(), key=string.lower):
        if pat and not pat.search(name):
            continue
        versions = c.list_versions(name)
        disp_name = name
        for version in versions:
            req = Req(name + ' ' + version)
            dist = c.get_dist(req)
            repo = dist_naming.repo_dist(dist)
            print fmt % (disp_name, version,  shorten_repo(repo))
            disp_name = ''


def read_depend_files():
    """
    Returns a dictionary mapping canonical project names to the spec
    dictionaries of the installed packages.
    """
    egg_info_dir = join(prefix, 'EGG-INFO')
    if not isdir(egg_info_dir):
        return {}
    res = {}
    for dn in os.listdir(egg_info_dir):
        path = join(egg_info_dir, dn, 'spec', 'depend')
        if isfile(path):
            spec = parse_data(open(path).read())
            add_Reqs_to_spec(spec)
            res[spec['cname']] = spec
    return res


def depend_warn(pkgs, ignore_version=False):
    """
    Warns the user about packages to be changed (i.e. removed or updated),
    if other packages depend on the package.

    Warnings are printed when the required name of the package matches.
    The ignore_version option determines if a version comparison is also
    desired as well, which it is not for the --remove option, since when
    a package is removed it does not matter which version is required.
    Hence, in remove_req() this function is called with ignore_version=True.
    """
    names = {}
    for pkg in pkgs:
        names[cname_fn(pkg)] = pkg
    index = read_depend_files()
    for spec in index.itervalues():
        if spec['cname'] in names:
            continue
        for req in spec["Reqs"]:
            if req.name not in names:
                continue
            if (ignore_version or
                     (req.version and
                      req.version != names[req.name].split('-')[1])):
                print "Warning: %s depends on %s" % (spec_as_req(spec), req)


def remove_req(req):
    """
    Tries remove a package from prefix given a requirement object.
    This function is only used for the --remove option.
    """
    d = get_installed_info(prefix, req.name)
    if not d:
        print "Package %r does not seem to be installed." % req.name
        return
    pkg = d['egg_name'][:-4]
    if req.version:
        v_a, b_a = pkg.split('-')[1:3]
        if req.version != v_a or (req.build and req.build != int(b_a)):
            print("Version mismatch: %s is installed cannot remove %s." %
                  (pkg, req))
            return
    depend_warn([pkg], ignore_version=True)
    egginst_remove(pkg)


def check_available(cname):
    avail = get_available()
    if cname not in avail:
        return False
    print """
But wait, %r is available in the EPD subscriber repository!
Would you like to go to %r
to subscribe?
""" % (cname, config.upgrade_epd_url)
    answer = raw_input('[yes|no]> ').strip().lower()
    if answer not in ('y', 'yes'):
        return False
    print """
Once you have obtained a subscription, you can proceed here.
"""
    import webbrowser
    webbrowser.open(config.upgrade_epd_url)
    config.write()
    return True


def get_dists(req, mode):
    """
    resolve the requirement
    """
    dists = c.install_sequence(req, mode)
    if dists:
        if verbose:
            print "Distributions in install sequence:"
            for d in dists:
                print '    ' + d
        return dists

    print "No distribution found for requirement '%s'." % req
    versions = c.list_versions(req.name)
    if versions:
        print "Versions for package %r are: %s" % (req.name,
                                                   ', '.join(versions))
    info = get_installed_info(prefix, req.name)
    if info:
        print "%(egg_name)s was installed on: %(mtime)s" % info
    elif 'EPD_free' in sys.version:
        if check_available(req.name):
            set_chain()
            return get_dists(req, mode)
    sys.exit(1)


def add_url(url):
    url = dist_naming.cleanup_reponame(url)

    arch_url = config.arch_filled_url(url)
    Chain([arch_url], verbose)

    if arch_url in config.get('IndexedRepos'):
        print "Already configured:", url
        return

    config.prepend_url(url)


def revert(rev_in):
    try:
        rev = int(rev_in)
    except ValueError:
        # we have a "date string"
        from parse_dt import parse
        rev = parse(rev_in)
        if rev is None:
            sys.exit("Error: could not parse: %r" % rev_in)

    print "reverting to: %r" % rev
    try:
        state = history.get_state(rev)
    except IndexError:
        sys.exit("Error: no such revision: %r" % rev)

    curr = set(egginst.get_installed())
    if state == curr:
        print "Nothing to revert"
        return

    # remove packages
    for fn in curr - state:
        egginst_remove(fn)

    # install packages (fetch from server if necessary)
    to_install = []
    need_fetch = []
    for fn in state - curr:
        to_install.append(fn)
        if not isfile(join(config.get('local'), fn)):
            need_fetch.append(fn)
    if need_fetch:
        set_chain()
        for fn in need_fetch:
            dist = c.get_dist(filename_as_req(fn))
            if dist:
                c.fetch_dist(dist, config.get('local'), dry_run=dry_run)
    for fn in to_install:
        pprint_fn_action(fn, 'installing')
        egg_path = join(config.get('local'), fn)
        if isfile(egg_path):
            ei = egginst.EggInst(egg_path)
            ei.install()

    history.update()


def iter_dists_excl(dists, exclude_fn):
    """
    Iterates over all dists, excluding the ones whose filename is an element
    of exclude_fn.  Yields the distribution.
    """
    for dist in dists:
        fn = dist_naming.filename_dist(dist)
        if fn in exclude_fn:
            continue
        yield dist


def install_req(req, opts):
    dists = get_dists(req, 'root' if opts.no_deps else 'recur')

    # warn the user about packages which depend on what will be updated
    depend_warn([dist_naming.filename_dist(d) for d in dists])

    # packages which are installed currently
    sys_inst = set(egginst.get_installed(sys.prefix))
    if prefix == sys.prefix:
        prefix_inst = sys_inst
    else:
        prefix_inst = set(egginst.get_installed(prefix))
    all_inst = sys_inst | prefix_inst

    # these are the packahes which are being excluded from being installed
    if opts.forceall:
        exclude = set()
    else:
        exclude = all_inst
        if opts.force:
            exclude.discard(dist_naming.filename_dist(dists[-1]))

    # fetch distributions
    if not isdir(config.get('local')):
        os.makedirs(config.get('local'))
    for dist in iter_dists_excl(dists, exclude):
        c.fetch_dist(dist, config.get('local'),
                     check_md5=opts.force or opts.forceall,
                     dry_run=dry_run)

    # remove packages (in reverse install order)
    for dist in dists[::-1]:
        fn = dist_naming.filename_dist(dist)
        if fn in all_inst:
            # if the distribution (which needs to be installed) is already
            # installed don't remove it
            continue
        cname = cname_fn(fn)
        # only remove packages installed in prefix
        for fn_inst in prefix_inst:
            if cname == cname_fn(fn_inst):
                egginst_remove(fn_inst)

    # install packages
    installed_something = False
    for dist in iter_dists_excl(dists, exclude):
        installed_something = True
        egginst_install(dist)

    if not installed_something:
        print "No update necessary, %s is up-to-date." % req
        print_installed_info(req.name)


def main():
    p = OptionParser(usage="usage: %prog [options] [name] [version]",
                     description=__doc__)

    p.add_option("--add-url",
                 action="store",
                 help="add a repository URL to the configuration file",
                 metavar='URL')

    p.add_option("--config",
                 action="store_true",
                 help="display the configuration and exit")

    p.add_option('-f', "--force",
                 action="store_true",
                 help="force install the main package "
                      "(not it's dependencies, see --forceall)")

    p.add_option("--forceall",
                 action="store_true",
                 help="force install of all packages "
                      "(i.e. including dependencies)")

    p.add_option('-i', "--info",
                 action="store_true",
                 help="show information about a package")

    p.add_option("--log",
                 action="store_true",
                 help="print revision log")

    p.add_option('-l', "--list",
                 action="store_true",
                 help="list the packages currently installed on the system")

    p.add_option('-n', "--dry-run",
                 action="store_true",
                 help="show what would have been downloaded/removed/installed")

    p.add_option('-N', "--no-deps",
                 action="store_true",
                 help="neither download nor install dependencies")

    p.add_option("--path",
                 action="store_true",
                 help="based on the configuration, display how to set the "
                      "PATH and PYTHONPATH environment variables")

    p.add_option("--prefix",
                 action="store",
                 help="install prefix (disregarding of any settings in "
                      "the config file)",
                 metavar='PATH')

    p.add_option("--proxy",
                 action="store",
                 help="use a proxy for downloads",
                 metavar='URL')

    p.add_option("--remove",
                 action="store_true",
                 help="remove a package")

    p.add_option("--revert",
                 action="store",
                 help="revert to a previous set of packages",
                 metavar="REV")

    p.add_option('-s', "--search",
                 action="store_true",
                 help="search the index in the repo (chain) of packages "
                      "and display versions available.")

    p.add_option("--sys-config",
                 action="store_true",
                 help="use <sys.prefix>/.enstaller4rc (even when "
                      "~/.enstaller4rc exists")

    p.add_option("--sys-prefix",
                 action="store_true",
                 help="use sys.prefix as the install prefix")

    p.add_option("--userpass",
                 action="store_true",
                 help="change EPD authentication in configuration file")

    p.add_option('-v', "--verbose", action="store_true")

    p.add_option('--version', action="store_true")

    p.add_option("--whats-new",
                 action="store_true",
                 help="display to which installed packages updates are "
                      "available")

    opts, args = p.parse_args()

    if len(args) > 0 and (opts.config or opts.path or opts.userpass or
                          opts.revert or opts.log or opts.whats_new):
        p.error("Option takes no arguments")

    if opts.prefix and opts.sys_prefix:
        p.error("Options --prefix and --sys-prefix exclude each ohter")

    if opts.force and opts.forceall:
        p.error("Options --force and --forceall exclude each ohter")

    pat = None
    if (opts.list or opts.search) and args:
        pat = re.compile(args[0], re.I)

    if opts.version:                              #  --version
        from enstaller import __version__
        print "enstaller version:", __version__
        return

    if opts.log:                                  #  --log
        history.print_log()
        return

    if opts.sys_config:                           #  --sys-config
        config.get_path = lambda: config.system_config_path

    if opts.config:                               #  --config
        config.print_config()
        return

    if opts.userpass:                             #  --userpass
        config.change_auth()
        return

    if opts.proxy:                                #  --proxy
        setup_proxy(opts.proxy)
    elif config.get('proxy'):
        setup_proxy(config.get('proxy'))
    else:
        setup_proxy()

    global prefix, dry_run, verbose    #  set globals
    if opts.sys_prefix:
        prefix = sys.prefix
    elif opts.prefix:
        prefix = opts.prefix
    else:
        prefix = config.get('prefix')
    dry_run = opts.dry_run
    verbose = opts.verbose

    if opts.add_url:                              #  --add-url
        add_url(opts.add_url)
        return

    if opts.path:                                 #  --path
        print_path()
        return

    if opts.list:                                 #  --list
        list_option(pat)
        return

    if opts.revert:                               #  --revert
        revert(opts.revert)
        return

    set_chain()                                   #  init chain

    if opts.search:                               #  --search
        search(pat)
        return

    if opts.info:                                 #  --info
        if len(args) != 1:
            p.error("Option requires one argument (name of package)")
        info_option(canonical(args[0]))
        return

    if opts.whats_new:                            # --whats-new
        whats_new()
        return

    if len(args) == 0:
        p.error("Requirement(s) missing")
    elif len(args) == 2:
        pat = re.compile(r'\d+\.\d+')
        if pat.match(args[1]):
            args = ['-'.join(args)]

    reqs = []
    for arg in args:
        if '-' in arg:
            name, version = arg.split('-', 1)
            reqs.append(Req(name + ' ' + version))
        else:
            reqs.append(Req(arg))

    if verbose:
        print "Requirements:"
        for req in reqs:
            print '    %r' % req
        print

    print "prefix:", prefix
    check_write()
    history.init()
    for req in reqs:
        if opts.remove:                           #  --remove
            remove_req(req)
        else:
            install_req(req, opts)

    history.update()


if __name__ == '__main__':
    main()



#------------------------------------------------------------------------------
# Copyright (c) 2007-2008 by Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license
# available at http://www.enthought.com/licenses/BSD.txt and may be
# redistributed only under the conditions described in the aforementioned
# license.
#
# urllib2 handlers for proxying SSL web requests using the CONNECT method.
#------------------------------------------------------------------------------


import base64
import httplib
import socket
import urllib
import urllib2


class ProxyHTTPConnection(httplib.HTTPConnection):

    _ports = {'http':80, 'https':443}


    def connect(self):
        """
        Open the network connection for the request.

        Overridden to setup a tunnel for the request through the proxy via a
        CONNECT request and response.  This follows the description of RFC2817
        as described here: ftp://ftp.isi.edu/in-notes/rfc2817.txt

        """

        # Initiate the actual network connection.
        httplib.HTTPConnection.connect(self)

        # Send the CONNECT message to the proxy.
        msg = self._build_connect_message()
        self.send(msg)

        # Read the proxy's response.  Note that closing the response does NOT
        # close the socket.
        response = self.response_class(self.sock, strict=self.strict,
            method=self._method, debuglevel=self.debuglevel)
        version, status, reason = response._read_status()
        while True:
            line = response.fp.readline()
            if self.debuglevel > 0:
                print "reply:", repr(line)
            if line == '\n' or line == '\r\n':
                break
        response.close()

        # If we can't communicate with the proxy, we have to completely punt
        # on the request.
        if status != 200:
            self.close()
            raise socket.error("Proxy failure: %d %s" % (status, reason))

        return


    def _build_connect_message(self):
        """
        Return a CONNECT message to be sent to the proxy.

        """

        # Retrieve the real target host and port we want to contact.
        authority = self._get_real_authority()

        # We always need the following headers.
        headers = {
            'Proxy-Connection': 'keep-alive',
            'Host': authority,
            'User-Agent': 'Python/2.5',
            }

        # If we have credentials to authenticate to the proxy, then proactively
        # provide the appropriate header.
        if self._proxy_info['pass'] is not None:
            raw = "%(user)s:%(pass)s" % self._proxy_info
            auth = 'Basic %s' % base64.encodestring(raw).strip()
            headers['Proxy-Authorization'] = auth

        # Assemble the message content to be sent
        buffer = ['CONNECT %s HTTP/1.0' % authority]
        for k, v in headers.items():
            buffer.append('%s: %s' % (k, v))
        buffer.extend(("", ""))
        msg = "\r\n".join(buffer)

        return msg


    def _get_real_authority(self):
        """
        Return the authority specification of the originally requested URL.

        The return value is a string of the form <host>:<port>.

        """

        url = self._proxy_request.get_selector()

        proto, rest = urllib.splittype(url)
        if proto is None:
            raise ValueError("unknown URL type: %s" % url)

        # Get the host and port specification
        host, rest = urllib.splithost(rest)
        host, port = urllib.splitport(host)

        # If port is not defined, then try to get it from the protocol.
        if port is None:
            try:
                port = self._ports[proto]
            except KeyError:
                raise ValueError("unknown protocol for: %s" % url)

        return '%s:%d' % (host, port)


class ConnectHTTPHandler(urllib2.HTTPHandler):

    def __init__(self, info=None, debuglevel=0):

        self.proxy_info = info
        urllib2.HTTPHandler.__init__(self, debuglevel)

        return


    def do_open(self, http_class, req):

        if self.proxy_info is not None:
            proxy = r'%(host)s:%(port)d' % self.proxy_info
            req.set_proxy(proxy, 'http')

        # We want to have a reference to the request during the execution of
        # the connect method of our HTTPConnection.  As a result, we create an
        # instance now and then give the dp_open method a callable that returns
        # that instance rather than a class.  Isn't Python wonderful?!?
        conn = ProxyHTTPConnection(req.host)
        conn._proxy_request = req
        conn._proxy_info = self.proxy_info
        def get_connection(host, timeout=req.timeout):
            conn._set_hostport(host, None)
            return conn

        return urllib2.HTTPHandler.do_open(self, get_connection, req)



import re
from datetime import datetime, timedelta


time_fmt = '%Y-%m-%d %H:%M:%S'
iso_pat = re.compile(r'\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}')


def simple_delta(s):
    pat = re.compile(r'(\d+)\s*(\w+)(?:\s+ago)?$')
    if s == 'yesterday':
        s = '1 day'
    m = pat.match(s)
    if m is None:
        return None
    n = int(m.group(1))
    unit = m.group(2).rstrip('s')
    if unit in ('week', 'wk'):
        return timedelta(weeks=n)
    if unit in ('day', 'd'):
        return timedelta(days=n)
    if unit in ('hour', 'hr'):
        return timedelta(hours=n)
    if unit in ('minute', 'min'):
        return timedelta(minutes=n)
    if unit in ('second', 'sec'):
        return timedelta(seconds=n)
    return None


def parse(s, cur=None):
    """
    parse the string 's' which can be, e.g. '1 hour', '2 weeks',
    and return something like '2011-08-02 23:03:43', or return None, if
    the string could be parsed
    """
    s = s.lower().strip()
    if iso_pat.match(s):
        return s

    if cur is None:
        cur = datetime.now()

    d = simple_delta(s)
    if d is not None:
        return str(cur - d)[:19]

    return None


if __name__ == '__main__':
    ref = datetime(2011, 8, 2, 23, 3, 43)
    print parse(' 35 wk   ', ref)



import sys
import time
import subprocess
from os.path import isfile, join

from bottle import get, post, request, put, run, view, debug, route, static_file

from enstaller.main import get_installed_info, cname_fn, shorten_repo
import egginst


debug(True)

def get_installed(prefix, pat=None):
    results = []
    for fn in egginst.get_installed(prefix):
        if pat and not pat.search(fn[:-4]):
            continue
        lst = list(egginst.name_version_fn(fn))
        info = get_installed_info(prefix, cname_fn(fn))
        if info is None:
            lst.append('-')
        else:
            path = join(info['meta_dir'], '__enpkg__.txt')
            if isfile(path):
                d = {}
                execfile(path, d)
                lst.append(shorten_repo(d['repo']))
            else:
                lst.append('-')
        results.append(tuple(lst))
    return results


@get('/status')
@view('status2')
def main():
    return {'items': get_installed(sys.prefix)}


@get('/')
@view('update2')
def update():
    print "Called update"
    lst = []
    for i, (package, version, repo) in enumerate(get_installed(sys.prefix)):
        lst.append(('#fff' if i % 2 else '#ccc',
                    package, version, repo, 'install'))
    #print lst
    return {'items': lst}


@post('/action')
def action():
    print 'request.forms', request.forms.dict

#    subprocess.call(['enpkg

    return update()


@route('/static/:path#.+#')
def server_static(path):
    return static_file(path, root='.')


@get('/installed')
def installed():
    #return {
    #    'id': 'Package',
    #    'label': 'Package',
    #    'items': [dict(zip(['Package', 'Version', 'Repository'], pkg))
    #        for pkg in get_installed(sys.prefix)]
    #}
    return {'items': get_installed(sys.prefix)}


run(host='localhost', port=8080)



import re
import sys
import time
import bisect
import string
from os.path import isfile, join

import egginst


PATH = join(sys.prefix, 'enpkg.hist')
TIME_FMT = '%Y-%m-%d %H:%M:%S %Z'


def ensure_path():
    if not isfile(PATH):
        sys.exit('Error: log file %r not found' % PATH)


def init(force=False):
    """
    initialize the history file
    """
    if not force and isfile(PATH):
        return
    fo = open(PATH, 'w')
    fo.write(time.strftime("==> %s <==\n" % TIME_FMT))
    for eggname in egginst.get_installed():
        fo.write('%s\n' % eggname)
    fo.close()


def parse():
    """
    parse the history file and return a list of
    tuples(datetime strings, set of eggs/diffs)
    """
    ensure_path()
    res = []
    sep_pat = re.compile(r'==>\s*(.+?)\s*<==')
    for line in open(PATH):
        line = line.strip()
        if not line or line.startswith('#'):
            continue
        m = sep_pat.match(line)
        if m:
            dt = m.group(1)
            res.append((dt, set()))
        else:
            res[-1][1].add(line)
    return res


def is_diff(cont):
    return any(s.startswith(('-', '+')) for s in cont)


def construct_states():
    """
    return a list of tuples(datetime strings, set of eggs)
    """
    res = []
    for dt, cont in parse():
        if is_diff(cont):
            for s in cont:
                if s.startswith('-'):
                    cur.discard(s[1:])
                elif s.startswith('+'):
                    cur.add(s[1:])
                else:
                    raise Exception('Did not expect: %s' % s)
        else:
            cur = cont
        res.append((dt, cur.copy()))
    return res


def find_revision(times, dt):
    """
    given a list of (sorted) datetimes 'times', return the index corresponding
    to the time 'dt'
    """
    i = bisect.bisect(times, dt)
    if i == 0:
        return 0
    else:
        return i - 1


def get_state(arg=None):
    """
    return the state, i.e. the set of eggs, for a given revision or time,
    defaults to latest
    """
    times, pkgs = zip(*construct_states())
    if arg is None:
        i = -1
    elif isinstance(arg, str):
        i = find_revision(times, arg)
    elif isinstance(arg, int):
        i = arg
    else:
        raise Exception('Did not expect: %r' % arg)
    return pkgs[i]


def update():
    """
    update the history file (creating a new one if necessary)
    """
    init()
    last = get_state()
    curr = set(egginst.get_installed())
    if last == curr:
        return
    fo = open(PATH, 'a')
    fo.write(time.strftime("==> %s <==\n" % TIME_FMT))
    for fn in last - curr:
        fo.write('-%s\n' % fn)
    for fn in curr - last:
        fo.write('+%s\n' % fn)
    fo.close()


def print_diff(diff):
    added = {}
    removed = {}
    for s in diff:
        fn = s[1:]
        name, version = egginst.name_version_fn(fn)
        if s.startswith('-'):
            removed[name.lower()] = version
        elif s.startswith('+'):
            added[name.lower()] = version
    changed = set(added) & set(removed)
    for name in sorted(changed):
        print '     %s  (%s -> %s)' % (name, removed[name], added[name])
    for name in sorted(set(removed) - changed):
        print '    -%s-%s' % (name, removed[name])
    for name in sorted(set(added) - changed):
        print '    +%s-%s' % (name, added[name])


def print_log():
    ensure_path()
    for i, (dt, cont) in enumerate(parse()):
        print '%s  (rev %d)' % (dt, i)
        if is_diff(cont):
            print_diff(cont)
        else:
            for x in sorted(cont, key=string.lower):
                print '    %s' % x
        print


if __name__ == '__main__':
    update()
    print_log()
    #get_state()



# Copyright by Enthought, Inc.
# Author: Ilan Schnell <ischnell@enthought.com>

import re
import os
import sys
import platform
from os.path import isfile, join

from enstaller import __version__
from utils import PY_VER, abs_expanduser
import plat


config_fn = ".enstaller4rc"
home_config_path = abs_expanduser("~/" + config_fn)
system_config_path = join(sys.prefix, config_fn)

pypi_url = 'http://www.enthought.com/repo/pypi/eggs/'
info_url = 'http://www.enthought.com/epd/index-info.bz2'
upgrade_epd_url = 'http://www.enthought.com/epd/upgrade'

default = dict(
    info_url=info_url,
    prefix=sys.prefix,
    proxy=None,
    noapp=False,
    local=join(sys.prefix, 'LOCAL-REPO'),
    EPD_auth=None,
    EPD_userpass=None,
    IndexedRepos=[pypi_url + plat.subdir + '/'],
)


def get_path():
    """
    Return the absolute path to our config file.
    """
    if isfile(home_config_path):
        return home_config_path
    if isfile(system_config_path):
        return system_config_path
    return None


def input_auth():
    from getpass import getpass
    print """\
In order to access the EPD repository, please enter your
username and password, which you use to subscribe to EPD.
If you are not subscribed to EPD, just hit Return.
"""
    username = raw_input('Username: ').strip()
    if not username:
        return None
    for dummy in xrange(3):
        password = getpass('Password: ')
        password2 = getpass('Confirm password: ')
        if password == password2:
            userpass = username + ':' + password
            return userpass.encode('base64').strip()
    return None

RC_TMPL = """\
# enstaller configuration file
# ============================
#
# This file contains the default package repositories, and configuration,
# used by enstaller %(version)s for the Python %(py_ver)s environment:
#
#   sys.prefix = %(sys_prefix)r
#
# This file was created by initially running the enpkg command.

%(auth_section)s
# The enpkg command is searching for eggs in the list 'IndexedRepos'.
# When enpkg is searching for an egg, it tries to find it in the order
# of this list, and selects the first one that matches, ignoring
# repositories below.  Therefore the order of this list matters.
#
# Placeholders '{ARCH}' get substituted by 'amd64' or 'x86', depending
# on the architecture of the current interpreter.
#
# Notice also that only indexed repositories, i.e. HTTP directories which
# contain a file 'index-depend.bz2' (next to the eggs), can be listed here.
# For local repositories, the index file is optional.  Remember that on
# Windows systems the backslaches in the directory path need to escaped, e.g.:
# r'file://C:\\repository\\' or 'file://C:\\\\repository\\\\'
IndexedRepos = [
%(repo_section)s]

# The following variable is optional and, if provided, point to a URL which
# contains an index file with additional package information, such as the
# package home-page, license type, description.  The information is displayed
# by the --info option.
#info_url = 'http://www.enthought.com/epd/index-info.bz2'

# Install prefix (enpkg --prefix and --sys-prefix options overwrite this).
# When this variable is not provided, it will default to the value of
# sys.prefix (within the current interpreter running enpkg)
#prefix = %(sys_prefix)r

# When running enpkg behind a firewall it might be necessary to use a proxy
# to access the repositories.  The URL for the proxy can be set here.
# Note that the enpkg --proxy option will overwrite this setting.
%(proxy_line)s

# Uncommenting the next line will disable application menu item install.
# This only effects the few packages which install menu items,
# which as IPython.
#noapp = True
"""

def write(proxy=None):
    """
    write the config file
    """
    try:
        from custom_tools import repo_section
    except ImportError:
        repo_section = ''

    # If user is 'root', then always create the config file in sys.prefix,
    # otherwise in the user's HOME directory.
    if sys.platform != 'win32' and os.getuid() == 0:
        path = system_config_path
    else:
        path = home_config_path

    auth = input_auth()
    if auth:
        auth_section = """
# The EPD subscriber authentication is required to access the EPD repository.
# To change this setting, use the 'enpkg --userpass' command which will ask
# you for your username and password.
EPD_auth = %r
""" % auth
    else:
        auth_section = ''

    py_ver = PY_VER
    sys_prefix = sys.prefix
    version = __version__

    if proxy:
        proxy_line = 'proxy = %r' % proxy
    else:
        proxy_line = '#proxy = <proxy string>  # e.g. "123.0.1.2:8080"'

    fo = open(path, 'w')
    fo.write(RC_TMPL % locals())
    fo.close()
    print "Wrote configuration file:", path
    print 77 * '='
    clear_cache()


def change_auth():
    path = get_path()
    if path is None:
        write()
        return
    fi = open(path)
    data = fi.read()
    fi.close()
    auth = input_auth()
    if not auth:
        return
    pat = re.compile(r'^EPD_auth\s*=.*$', re.M)
    authline = 'EPD_auth = %r' % auth
    if pat.search(data):
        data = pat.sub(authline, data)
    else:
        lines = data.splitlines()
        lines.insert(10, authline)
        data = '\n'.join(lines) + '\n'
    fo = open(path, 'w')
    fo.write(data)
    fo.close()


def prepend_url(url):
    f = open(get_path(), 'r+')
    data = f.read()
    pat = re.compile(r'^IndexedRepos\s*=\s*\[\s*$', re.M)
    if not pat.search(data):
        sys.exit("Error: IndexedRepos section not found")
    data = pat.sub(r"IndexedRepos = [\n  '%s'," % url, data)
    f.seek(0)
    f.write(data)
    f.close()


def arch_filled_url(url):
    from indexed_repo.dist_naming import cleanup_reponame

    return cleanup_reponame(url.replace('{ARCH}', plat.arch))


def clear_cache():
    if hasattr(read, 'cache'):
        del read.cache


def read():
    """
    return the configuration from the config file as a dictionary,
    and fix some values and give defaults
    """
    if hasattr(read, 'cache'):
        return read.cache

    path = get_path()
    read.cache = dict(default)
    if path is None:
        return read()

    d = {}
    execfile(path, d)
    for k in default.iterkeys():
        if not d.has_key(k):
            continue
        v = d[k]
        if k == 'IndexedRepos':
            read.cache[k] = [arch_filled_url(url) for url in v]
        elif k in ('prefix', 'local'):
            read.cache[k] = abs_expanduser(v)
        else:
            read.cache[k] = v
    return read()


def get(key):
    return read()[key]


def print_config():
    print "Python version:", PY_VER
    print "enstaller version:", __version__
    print "sys.prefix:", sys.prefix
    print "platform:", platform.platform()
    print "architecture:", platform.architecture()[0]
    print "config file:", get_path()
    print
    print "settings:"
    for k in 'info_url', 'prefix', 'local', 'noapp', 'proxy':
        print "    %s = %r" % (k, get(k))
    print "    IndexedRepos:"
    for repo in get('IndexedRepos'):
        print '        %r' % repo


if __name__ == '__main__':
    write()
    print_config()



# this test code is not imported anywhere
import xmlrpclib
import sys

pypi = xmlrpclib.ServerProxy('http://python.org/pypi')
for hit in pypi.search(dict(name=sys.argv[1])):
    print '''\
NAME: %(name)s %(version)s
SUMM: %(summary)s''' % hit
    urls = pypi.package_urls(hit['name'], hit['version'])
    if urls:
        for url in urls:
            #print repr(url)
            print '''\
%(url)s
\t filename: %(filename)s (%(size)i bytes)
\t pkgtype : %(packagetype)s
\t py-vers : %(python_version)s''' % url

    print



sys.exit()

import re
import StringIO

import enstaller.utils as utils



URL = 'http://pypi.python.org/simple/'

if 0:
    faux = StringIO.StringIO()
    utils.write_data_from_url(faux, URL)
    data = faux.getvalue()

data = open('simple.txt').read()

pat = re.compile(r'<a href=.+?>(.+?)<')

for line in data.splitlines():
    m = pat.match(line)
    if m:
        print m.group(1)



#------------------------------------------------------------------------------
# Copyright (c) 2007 by Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license
# available at http://www.enthought.com/licenses/BSD.txt and may be
# redistributed only under the conditions described in the aforementioned
# license.
#------------------------------------------------------------------------------


import urllib2

from enthought.proxy.api import ConnectHTTPHandler, ConnectHTTPSHandler


if __name__ == '__main__':

    import sys

    # Point this at a working proxy server.  The below is specific to my
    # machine.
    p_info = {'host':'10.1.10.187', 'port':3128, 'user':'user', 'pass':'user'}

    # Setup urllib2 to connect through the proxy for http and https.
    opener = urllib2.build_opener(
        ConnectHTTPHandler(info=p_info),
        ConnectHTTPSHandler(info=p_info),
        )
    urllib2.install_opener(opener)

    # Try an HTTPS request
    f = urllib2.urlopen(r"https://svn.enthought.com/svn/enthought")
    print f.read()
    f.close()
    print "\n\n"

    # Try an HTTP request
    f = urllib2.urlopen(r"http://code.enthought.com/enstaller/eggs")
    print f.read()
    f.close()
    print "\n\n"




#------------------------------------------------------------------------------
# Copyright (c) 2007-2008 by Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license
# available at http://www.enthought.com/licenses/BSD.txt and may be
# redistributed only under the conditions described in the aforementioned
# license.
#
#------------------------------------------------------------------------------


import os
import urllib2

from connect_HTTP_handler import ConnectHTTPHandler
from connect_HTTPS_handler import ConnectHTTPSHandler


def install_proxy_handlers(pinfo):
    """
    Use a proxy for future urllib2.urlopen commands.

    The specified pinfo should be a dictionary containing the following:
        * host: the servername of the proxy
        * port: the port to use on the proxy server
        * user: (optional) username for authenticating with the proxy server.
        * pass: (optional) password for authenticating with the proxy server.

    """

    h = pinfo['host']
    p = pinfo['port']
    usr = pinfo['user']
    pwd = pinfo['pass']

    # Only install a custom opener if a host was actually specified.
    if h is not None and len(h) > 0:
        handlers = []

        # Add handlers to deal with using the proxy.
        handlers.append(ConnectHTTPSHandler(info=pinfo))
        handlers.append(ConnectHTTPHandler(info=pinfo))

        # Create a proxy opener and install it.
        opener = urllib2.build_opener(*handlers)
        urllib2.install_opener(opener)

    return


def get_proxy_info(proxystr=None):
    """
    Get proxy config from string or environment variables.

    If a proxy string is passed in, it overrides whatever might be in the
    environment variables.

    Returns dictionary of identified proxy information.

    Raises ValueError on any configuration error.

    """

    default_port = 80

    # Only check for env variables if no explicit proxy string was provided.
    if proxystr is None or len(proxystr) < 1:
        proxy_info = {
            'host' : os.environ.get('PROXY_HOST', None),
            'port' : os.environ.get('PROXY_PORT', default_port),
            'user' : os.environ.get('PROXY_USER', None),
            'pass' : os.environ.get('PROXY_PASS', None)
            }

    # Parse the passed proxy string
    else:
        proxy_info = {}
        res = proxystr.split('@')
        if len(res) == 1:
            user_pass = [None]
            host_port = res[0].split(':')
        elif len(res) == 2:
            user_pass = res[0].split(':')
            host_port = res[1].split(':')
        else:
            raise ValueError('Invalid proxy string: "%s"' % proxystr)

        if len(user_pass) == 1:
            proxy_info['user'] = user_pass[0]
            proxy_info['pass'] = None
        elif len(user_pass) == 2:
            proxy_info['user'] = user_pass[0]
            proxy_info['pass'] = user_pass[1]
        else:
            raise ValueError('Invalid user:pass in proxy string: '
                '"%s"' % user_pass)

        if len(host_port) == 1:
            proxy_info['host'] = host_port[0]
            proxy_info['port'] = default_port
        elif len(host_port) == 2:
            proxy_info['host'] = host_port[0]
            try:
                p = int(host_port[1])
            except:
                raise ValueError('Port specification must be an integer.  '
                    'Had "%s"' % host_port[1])
            proxy_info['port'] = p
        else:
            raise ValueError('Invalid host:port in proxy string: '
                '"%s"' % host_port)

    # If a user was specified, but no password was, prompt for it now.
    user = proxy_info.get('user', None)
    if user is not None and len(user) > 0:
        pwd = proxy_info.get('pass', None)
        if pwd is None or len(pwd) < 1:
            import getpass
            proxy_info['pass'] = getpass.getpass()

    return proxy_info


def setup_authentication(cfg, opener=None):
    """
    """

    # Configure a password manager with the user's authentication info.
    passmgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
    section = 'url auth info'
    if cfg.has_section(section):
        for dummy, info in cfg.items(section):

            # Ensure the info includes both a username and a url.
            if '@' not in info:
                raise ValueError('Invalid %s string: "%s"' % (section, info))
            userpass, url = info.split('@')

            # Ensure we have both a user and password.
            if ':' in userpass:
                user, password = userpass.split(':')
            else:
                user = userpass
                prompt = 'Password for %s@%s: ' % (user, url)
                import getpass
                password = getpass.getpass(prompt)

            passmgr.add_password(None, url, user, password)

    # Create a basic auth handler that uses our authentication info.
    handler = urllib2.HTTPBasicAuthHandler(passmgr)

    # Add to an existing opener if one was specified and otherwise, create and
    # register our own.
    if opener is not None:
        opener.add_handler(handler)
    else:
        opener = urllib2.build_opener(handler)
        urllib2.install_opener(opener)

    return


def setup_proxy(proxystr=''):
    """
    Configure and install proxy support.

    The specified proxy string is parsed via ``get_proxy_info`` and then
    installed via ``install_proxy_handler``.  If proxy settings are detected
    and a handler is installed, then this method returns True.  Otherwise it
    returns False.

    Raises ValueError in the event of any problems.

    """

    installed = False

    info = get_proxy_info(proxystr)
    if 'host' in info and info['host'] is not None:
        install_proxy_handlers(info)
        installed = True

    return installed



"""\
web-interface for enpkg command
"""
import sys
import time
import subprocess
import socket
from os.path import dirname, isfile, join

this_dir = dirname(__file__)
sys.path.insert(0, this_dir)

from bottle import get, post, request, run, view, debug, route, static_file

from api import get_status
import egginst


debug(True)

css_class_map = {
    'up-to-date': 'ok',
    'installed': 'ok',
    'updateable': 'up',
    'installable': 'inst',
}

@get('/')
@view(join(this_dir, 'update'))
def update():
    print "Called update"
    lst = []
    status = get_status()
    for cname in sorted(status.iterkeys()):
        d = status[cname]
        lst.append((
                css_class_map.get(d['status'], 'unknown'),
                d['name'], d['version'], d['a-ver'], d['status'],
                d['status'].endswith('able'),
        ))
    return {'items': lst}


@post('/action')
def action():
    print 'request.forms', request.forms.dict
    for name in request.forms.dict.iterkeys():
        subprocess.call(['enpkg', name])
    return update()


@route('/static/:path#.+#')
def server_static(path):
    return static_file(path, root=this_dir)


def create_static_html():
    import tempfile

    tmp_dir = tempfile.mkdtemp()
    index_html = join(tmp_dir, 'index.html')

    fo = open(index_html, 'w')
    fo.write("I'm a static file\n")
    fo.close()

    return index_html


def main():
    from optparse import OptionParser

    p = OptionParser(usage="usage: %prog [options]", description=__doc__)

    p.add_option('-b', "--browser",
                 action="store_true",
                 help="lauch a web-browser")

    p.add_option('-p', "--port",
                 action="store",
                 default=8080,
                 help="defaults to %default")

    opts, args = p.parse_args()

    port = int(opts.port)
    url = 'http://localhost:%d/' % port

    if '-b' in sys.argv:
        import webbrowser
        print 'opening in web-browser:', url
        webbrowser.open_new_tab(url)

#    try:
    run(host='localhost', port=port)
#    except socket.error:
#        print "Could not start web server, creating static html file"
#        url = 'file://%s' % create_static_html()


if __name__ == '__main__':
    main()



# Author: Ilan Schnell <ischnell@enthought.com>
"""\
egginst is a simple tool for installing and uninstalling eggs.  The tool
is brain dead in the sense that it does not care if the eggs it installs
are for the correct platform, it's dependencies got installed, another
package needs to be uninstalled prior to the install, and so on.  Those tasks
are responsibilities of a package manager, e.g. enpkg.  You just give it
eggs and it installs/uninstalls them.
"""
import os
import sys
import re
import zipfile
import ConfigParser
from os.path import abspath, basename, dirname, join, isdir, isfile

from utils import (on_win, bin_dir_name, rel_site_packages,
                   pprint_fn_action, rm_empty_dir, rm_rf, human_bytes)
import scripts



NS_PKG_PAT = re.compile(
    r'\s*__import__\([\'"]pkg_resources[\'"]\)\.declare_namespace'
    r'\(__name__\)\s*$')


def name_version_fn(fn):
    """
    Given the filename of a package, returns a tuple(name, version).
    """
    if fn.endswith('.egg'):
        fn = fn[:-4]
    if '-' in fn:
        return tuple(fn.split('-', 1))
    else:
        return fn, ''


class EggInst(object):

    def __init__(self, fpath, prefix=sys.prefix,
                 hook=False, verbose=False, noapp=False):
        self.fpath = fpath
        self.cname = name_version_fn(basename(fpath))[0].lower()
        self.prefix = abspath(prefix)
        self.hook = bool(hook)
        self.noapp = noapp

        self.bin_dir = join(self.prefix, bin_dir_name)

        if self.hook:
            self.pkgs_dir = join(self.prefix, 'pkgs')
            self.pkg_dir = join(self.pkgs_dir, basename(fpath)[:-4])
            self.pyloc = self.pkg_dir
            self.meta_dir = join(self.pkg_dir, 'EGG-INFO')
            self.registry_txt = join(self.meta_dir, 'registry.txt')
        else:
            self.site_packages = join(self.prefix, rel_site_packages)
            self.pyloc = self.site_packages
            self.egginfo_dir = join(self.prefix, 'EGG-INFO')
            self.meta_dir = join(self.egginfo_dir, self.cname)

        self.meta_txt = join(self.meta_dir, '__egginst__.txt')
        self.files = []
        self.verbose = verbose

    def rel_prefix(self, path):
        assert abspath(path).startswith(self.prefix)
        return path[len(self.prefix) + 1:]


    def install(self):
        if not isdir(self.meta_dir):
            os.makedirs(self.meta_dir)

        self.z = zipfile.ZipFile(self.fpath)
        self.arcnames = self.z.namelist()

        self.extract()

        if on_win:
            scripts.create_proxies(self)

        else:
            import links
            import object_code

            if self.verbose:
                links.verbose = object_code.verbose = True

            links.create(self)
            object_code.fix_files(self)

        self.entry_points()
        self.z.close()
        scripts.fix_scripts(self)
        self.run('post_egginst.py')
        self.install_app()
        self.write_meta()

        if self.hook:
            import registry

            registry.create_file(self)


    def entry_points(self):
        lines = list(self.lines_from_arcname('EGG-INFO/entry_points.txt',
                                             ignore_empty=False))
        if lines == []:
            return

        path = join(self.meta_dir, '__entry_points__.txt')
        fo = open(path, 'w')
        fo.write('\n'.join(lines) + '\n')
        fo.close()

        conf = ConfigParser.ConfigParser()
        conf.read(path)
        if ('console_scripts' in conf.sections() or
            'gui_scripts' in conf.sections()):
            if self.verbose:
                print 'creating scripts'
                scripts.verbose = True
            scripts.create(self, conf)


    def write_meta(self):
        fo = open(self.meta_txt, 'w')
        fo.write('# egginst metadata\n')
        fo.write('egg_name = %r\n' % basename(self.fpath))
        fo.write('prefix = %r\n' % self.prefix)
        fo.write('installed_size = %i\n' % self.installed_size)
        fo.write('rel_files = [\n')
        fo.write('  %r,\n' % self.rel_prefix(self.meta_txt))
        for p in self.files:
            if abspath(p).startswith(self.prefix):
                fo.write('  %r,\n' % self.rel_prefix(p))
            else:
                fo.write('  %r,\n' % p)
        fo.write(']\n')
        fo.close()

    def read_meta(self):
        d = {'installed_size': -1}
        execfile(self.meta_txt, d)
        for name in ['egg_name', 'prefix', 'installed_size', 'rel_files']:
            setattr(self, name, d[name])
        self.files = [join(self.prefix, f) for f in d['rel_files']]


    def lines_from_arcname(self, arcname,
                           ignore_empty=True,
                           ignore_comments=True):
        if not arcname in self.arcnames:
            return
        for line in self.z.read(arcname).splitlines():
            line = line.strip()
            if ignore_empty and line == '':
                continue
            if ignore_comments and line.startswith('#'):
                continue
            yield line


    def extract(self):
        cur = n = 0
        size = sum(self.z.getinfo(name).file_size for name in self.arcnames)
        sys.stdout.write('%9s [' % human_bytes(size))
        for name in self.arcnames:
            n += self.z.getinfo(name).file_size
            if size == 0:
                rat = 1
            else:
                rat = float(n) / size
            if rat * 64 >= cur:
                sys.stdout.write('.')
                sys.stdout.flush()
                cur += 1
            self.write_arcname(name)

        self.installed_size = size
        sys.stdout.write('.' * (65 - cur) + ']\n')
        sys.stdout.flush()


    def get_dst(self, arcname):
        if (not self.hook and arcname == 'EGG-INFO/PKG-INFO' and
                      self.fpath.endswith('.egg')):
            return join(self.site_packages, basename(self.fpath) + '-info')

        for start, cond, dst_dir in [
            ('EGG-INFO/prefix/',  True,       self.prefix),
            ('EGG-INFO/usr/',     not on_win, self.prefix),
            ('EGG-INFO/scripts/', True,       self.bin_dir),
            ('EGG-INFO/',         True,       self.meta_dir),
            ('',                  True,       self.pyloc),
            ]:
            if arcname.startswith(start) and cond:
                return abspath(join(dst_dir, arcname[len(start):]))
        raise Exception("Didn't expect to get here")

    py_pat = re.compile(r'^(.+)\.py(c|o)?$')
    so_pat = re.compile(r'^lib.+\.so')
    py_obj = '.pyd' if on_win else '.so'
    def write_arcname(self, arcname):
        if arcname.endswith('/') or arcname.startswith('.unused'):
            return
        m = self.py_pat.match(arcname)
        if m and (m.group(1) + self.py_obj) in self.arcnames:
            # .py, .pyc, .pyo next to .so are not written
            return
        path = self.get_dst(arcname)
        dn, fn = os.path.split(path)
        data = self.z.read(arcname)
        if fn in ['__init__.py', '__init__.pyc']:
            tmp = arcname.rstrip('c')
            if tmp in self.arcnames and NS_PKG_PAT.match(self.z.read(tmp)):
                if fn == '__init__.py':
                    data = ''
                if fn == '__init__.pyc':
                    return
        self.files.append(path)
        if not isdir(dn):
            os.makedirs(dn)
        rm_rf(path)
        fo = open(path, 'wb')
        fo.write(data)
        fo.close()
        if (arcname.startswith(('EGG-INFO/usr/bin/', 'EGG-INFO/scripts/')) or
                fn.endswith(('.dylib', '.pyd', '.so')) or
                (arcname.startswith('EGG-INFO/usr/lib/') and
                 self.so_pat.match(fn))):
            os.chmod(path, 0755)


    def install_app(self, remove=False):
        if self.noapp:
            return

        path = join(self.meta_dir, 'inst', 'appinst.dat')
        if not isfile(path):
            return

        try:
            import appinst
        except ImportError:
            return

        try:
            if remove:
                appinst.uninstall_from_dat(path)
            else:
                appinst.install_from_dat(path)
        except Exception, e:
            print("Warning (%sinstalling application item):\n%r" %
                  ('un' if remove else '', e))


    def run(self, fn):
        path = join(self.meta_dir, fn)
        if not isfile(path):
            return
        from subprocess import call
        call([sys.executable, '-E', path, '--prefix', self.prefix],
             cwd=dirname(path))


    def rm_dirs(self):
        dir_paths = set()
        len_prefix = len(self.prefix)
        for path in set(dirname(p) for p in self.files):
            while len(path) > len_prefix:
                dir_paths.add(path)
                path = dirname(path)

        for path in sorted(dir_paths, key=len, reverse=True):
            rm_empty_dir(path)

    def remove(self):
        if not isdir(self.meta_dir):
            print "Error: Can't find meta data for:", self.cname
            return

        self.read_meta()
        cur = n = 0
        nof = len(self.files) # number of files
        sys.stdout.write('%9s [' % human_bytes(self.installed_size))
        self.install_app(remove=True)
        self.run('pre_egguninst.py')

        for p in self.files:
            n += 1
            rat = float(n) / nof
            if rat * 64 >= cur:
                sys.stdout.write('.')
                sys.stdout.flush()
                cur += 1
            rm_rf(p)
            if p.endswith('.py'):
                rm_rf(p + 'c')
        self.rm_dirs()
        rm_rf(self.meta_dir)
        if self.hook:
            rm_empty_dir(self.pkg_dir)
        else:
            rm_empty_dir(self.egginfo_dir)
        sys.stdout.write('.' * (65 - cur) + ']\n')
        sys.stdout.flush()


def get_installed_cnames(prefix=sys.prefix):
    """
    returns a sorted list of cnames of all installed packages
    """
    egg_info_dir = join(prefix, 'EGG-INFO')
    if not isdir(egg_info_dir):
        return []
    pat = re.compile(r'([a-z0-9_.]+)$')
    return sorted(fn for fn in os.listdir(egg_info_dir) if pat.match(fn))


def get_installed(prefix=sys.prefix):
    """
    Generator returns a sorted list of all installed packages.
    Each element is the filename of the egg which was used to install the
    package.
    """
    egg_info_dir = join(prefix, 'EGG-INFO')
    for cname in get_installed_cnames(prefix):
        meta_txt = join(egg_info_dir, cname, '__egginst__.txt')
        if not isfile(meta_txt):
            continue
        d = {}
        execfile(meta_txt, d)
        yield d['egg_name']


def print_installed(prefix=sys.prefix):
    fmt = '%-20s %s'
    print fmt % ('Project name', 'Version')
    print 40 * '='
    for fn in get_installed(prefix):
        print fmt % name_version_fn(fn)


def main():
    from optparse import OptionParser

    p = OptionParser(usage="usage: %prog [options] [EGGS ...]",
                     description=__doc__)

    p.add_option('-l', "--list",
                 action="store_true",
                 help="list all installed packages")

    p.add_option("--noapp",
                 action="store_true",
                 help="don't install/remove application menu items")

    p.add_option("--prefix",
                 action="store",
                 default=sys.prefix,
                 help="install prefix, defaults to %default",
                 metavar='PATH')

    p.add_option("--hook",
                 action="store_true",
                 help="don't install into site-packages (experimental)")

    p.add_option('-r', "--remove",
                 action="store_true",
                 help="remove package(s), requires the egg or project name(s)")

    p.add_option('-v', "--verbose", action="store_true")
    p.add_option('-n', "--dry-run", action="store_true")
    p.add_option('--version', action="store_true")

    opts, args = p.parse_args()

    if opts.version:
        from enstaller import __version__
        print "enstaller version:", __version__
        return

    prefix = abspath(opts.prefix)

    if opts.list:
        if args:
            p.error("the --list option takes no arguments")
        print_installed(prefix)
        return

    for path in args:
        ei = EggInst(path, prefix, opts.hook, opts.verbose, opts.noapp)
        fn = basename(path)
        if opts.remove:
            pprint_fn_action(fn, 'removing')
            if opts.dry_run:
                continue
            ei.remove()

        else: # default is always install
            pprint_fn_action(fn, 'installing')
            if opts.dry_run:
                continue
            ei.install()


if __name__ == '__main__':
    main()



"""
This is the API for egginst
===========================

More text here...


Egginst class
-------------

Is instantiated by an argument ARG (see below) and an optional verbose
keyword argument and has the following methods (for public use):

install():
    Installs the egg, provided by ARG, into the current Python environment

remove():
    Remove an installed package.  The ARG may be:
       * the full path of the egg which was used during the install
       * the egg name
       * or simply the project name


Functions:
----------

get_installed(prefix):
    Generator returns a sorted list of all installed packages.
    Each element is the filename of the egg which was used to install the
    package.
"""
from egginst.main import (EggInst, get_installed_cnames, get_installed,
                          name_version_fn)



from distutils.util import convert_path

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

kwds = {} # Additional keyword arguments for setup

d = {}
execfile(convert_path('enstaller/__init__.py'), d)
kwds['version'] = d['__version__']

f = open('README.rst')
kwds['long_description'] = f.read()
f.close()


setup(
    name="enstaller",
    author="Enthought, Inc.",
    author_email="info@enthought.com",
    url = "https://github.com/enthought/enstaller",
    license="BSD",
    description = "Install and managing tool for egg-based packages",
    packages = [
        'egginst',
        'enstaller',
        'enstaller/indexed_repo',
        'enstaller/proxy',
    ],
    entry_points = {
        "console_scripts": [
             "enpkg = enstaller.main:main",
             "egginst = egginst.main:main",
        ],
    },
    classifiers = [
        "License :: OSI Approved :: BSD License",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: System :: Software Distribution",
        "Topic :: System :: Systems Administration",
    ],
    **kwds
)



#------------------------------------------------------------------------------
# Copyright (c) 2007-2008 by Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license
# available at http://www.enthought.com/licenses/BSD.txt and may be
# redistributed only under the conditions described in the aforementioned
# license.
#------------------------------------------------------------------------------


from connect_HTTP_handler import ConnectHTTPHandler
from connect_HTTPS_handler import ConnectHTTPSHandler
from util import install_proxy_handlers, setup_authentication, setup_proxy




import sys
import os
import shutil
from os.path import isdir, isfile, islink


on_win = bool(sys.platform == 'win32')

if on_win:
    bin_dir_name = 'Scripts'
    rel_site_packages = r'Lib\site-packages'
else:
    bin_dir_name = 'bin'
    rel_site_packages = 'lib/python%i.%i/site-packages' % sys.version_info[:2]


def pprint_fn_action(fn, action):
    """
    Pretty print the distribution name (filename) and an action, the width
    of the output corresponds to the with of the progress bar used by the
    function below.
    """
    print "%-56s %20s" % (fn, '[%s]' % action)


def rm_empty_dir(path):
    """
    Remove the directory `path` if it is a directory and empty.
    If the directory does not exist or is not empty, do nothing.
    """
    try:
        os.rmdir(path)
    except OSError: # directory might not exist or not be empty
        pass


def rm_rf(path, verbose=False):
    if not on_win and islink(path):
        # Note that we have to check if the destination is a link because
        # exists('/path/to/dead-link') will return False, although
        # islink('/path/to/dead-link') is True.
        if verbose:
            print "Removing: %r (link)" % path
        os.unlink(path)

    elif isfile(path):
        if verbose:
            print "Removing: %r (file)" % path
        if sys.platform == 'win32':
            try:
                os.unlink(path)
            except WindowsError:
                pass
        else:
            os.unlink(path)

    elif isdir(path):
        if verbose:
            print "Removing: %r (directory)" % path
        shutil.rmtree(path)


def human_bytes(n):
    """
    Return the number of bytes n in more human readable form.
    """
    if n < 1024:
        return '%i B' % n

    k = (n - 1) / 1024 + 1
    if k < 1024:
        return '%i KB' % k

    return '%.2f MB' % (float(n) / (2**20))



#------------------------------------------------------------------------------
# Copyright (c) 2007-2008 by Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license
# available at http://www.enthought.com/licenses/BSD.txt and may be
# redistributed only under the conditions described in the aforementioned
# license.
#
# urllib2 handlers for proxying SSL web requests using the CONNECT method.
#------------------------------------------------------------------------------


import base64
import httplib
import socket
import urllib2

from connect_HTTP_handler import ProxyHTTPConnection


class ProxyHTTPSConnection(ProxyHTTPConnection):

    default_port = 443

    def __init__(self, host, port = None, key_file = None, cert_file = None,
        strict = None):

        ProxyHTTPConnection.__init__(self, host, port)
        self.key_file = key_file
        self.cert_file = cert_file

        return


    def connect(self):

        ProxyHTTPConnection.connect(self)

        # Make the socket ssl-aware
        ssl = socket.ssl(self.sock, self.key_file, self.cert_file)
        self.sock = httplib.FakeSocket(self.sock, ssl)

        return


class ConnectHTTPSHandler(urllib2.HTTPSHandler):

    def __init__(self, info=None, debuglevel=0):

        self.proxy_info = info
        urllib2.HTTPSHandler.__init__(self, debuglevel)

        return


    def do_open(self, http_class, req):

        if self.proxy_info is not None:
            proxy = r'%(host)s:%(port)d' % self.proxy_info
            req.set_proxy(proxy, 'https')

        # We want to have a reference to the request during the execution of
        # the connect method of our HTTPConnection.  As a result, we create an
        # instance now and then give the dp_open method a callable that returns
        # that instance rather than a class.  Isn't Python wonderful?!?
        conn = ProxyHTTPSConnection(req.host)
        conn._proxy_request = req
        conn._proxy_info = self.proxy_info
        def get_connection(host):
            conn._set_hostport(host, None)
            return conn

        return urllib2.HTTPSHandler.do_open(self, get_connection, req)




import sys
from os.path import abspath, dirname

from enstaller.indexed_repo import Chain
from enstaller.indexed_repo.requirement import Req



def show(dists):
    if dists is None:
        print 'none'
        return
    for d in dists:
        print d
    print


c = Chain(verbose=1)
for name in ('runner', 'epd'):
    repo = 'file://%s/%s/' % (abspath(dirname(__file__)), name)
    c.add_repo(repo, 'index-7.1.txt')


req = Req(' '.join(sys.argv[1:]))
show(c.install_sequence(req, 'flat'))



from distutils.util import convert_path

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


setup(
    name="enpkg_server",
    version="1.0",
    author="Enthought, Inc.",
    author_email="info@enthought.com",
    url = "https://github.com/enthought/enstaller",
    license="BSD",
    description = "Install and managing tool for egg-based packages",
    packages = [
        'enpkg_server',
    ],
    entry_points = {
        "console_scripts": [
             "enpkg-server = enpkg_server.main:main",
        ],
    },
)
